/*
 * Copyright (C) 2022 Camden Dixie O'Brien
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdbool.h>
#include <stdio.h>
#include <sys/socket.h>
#include <unistd.h>

static bool exit_requested = false;

void handle_exit_signal(int signum)
{
	(void)signum;
	exit_requested = true;
}

int main(int argc, char *argv[])
{
	(void)argc;
	(void)argv;

	if (signal(SIGTERM, handle_exit_signal) == SIG_ERR) {
		fprintf(stderr, "Failed to register SIGTERM signal handler\n");
		return 1;
	}
	if (signal(SIGINT, handle_exit_signal) == SIG_ERR) {
		fprintf(stderr, "Failed to register SIGINT signal handler\n");
		return 1;
	}

	int sfd = socket(AF_INET6, SOCK_STREAM, 0);
	if (sfd == -1) {
		fprintf(stderr, "Failed to open socket\n");
		return 1;
	}

	const struct sockaddr_in6 haddr = {
		.sin6_family = AF_INET6,
		.sin6_port = htons(7070),
		.sin6_addr = IN6ADDR_LOOPBACK_INIT,
	};
	if (bind(sfd, (const struct sockaddr *)&haddr, sizeof(haddr)) == -1) {
		fprintf(stderr, "Error binding socket to address\n");
		return 1;
	}
	if (listen(sfd, 8) == -1) {
		fprintf(stderr, "Error attempting to listen on socket\n");
		return 1;
	}

	struct sockaddr_in6 paddr;
	socklen_t paddr_size = sizeof(paddr);
	int cfd;
	while (!exit_requested) {
		cfd = accept(sfd, (struct sockaddr *)&paddr, &paddr_size);
		if (cfd == -1) {
			if (errno == EINTR)
				continue;
			fprintf(stderr, "Error accepting connection\n");
			return 1;
		}

		printf("Accepted connection\n");

		close(cfd);
	}

	close(sfd);

	return 0;
}
